import QtQuick 1.1
import com.nokia.meego 1.0
import QtMultimediaKit 1.1
import "storage.js" as Storage
import TouchEvents 1.0


Page {
    id: gamepage
    tools: commonTools

    property int lives: 3;
    property int tailoffset: 0;
    property int score: 0;
    property int stars: 0;

    property int delayStar: 0;

    property int delay1: 0
    property int speed1: 0;

    property int delay2: 0;
    property int speed2: 0;
    property real random: 0;

    property bool gameplaying: false;
    property bool gamepaused: false;
    property int highscore;

    orientationLock: PageOrientation.LockLandscape;


    function startGame() {
        lives = 3;
        score = 0;
        stars = 0;
        speed1 = 0;
        speed2 = 0;

        asteroid.x = 854;
        asteroid2.x = 854;
        star.x = 854;

        nyancat.y = 180

        stars1.x = -854;
        stars2.x = 0;

        musicSound.play();

        nyanCatAnim.start();

        star.x = 854;

        gameOverLabel.visible = false;

        newHighscoreLabel.visible = true;

        Storage.initialize();
        highscore = Storage.getInt("highscore");
        console.log("higscore: " + highscore);
    }


    onGamepausedChanged: {
        if (gamepaused)
            gamePausedAdnim.start();
        else {
            gamePausedAdnim.stop();
            gamePausedLabel.opacity = 0;
        }
        screen.allowSwipe = gamepaused;
        if (musicSound.playing == false) {
            if (gamepaused)
                musicSound2.stop();
            else
                musicSound2.play();
        }
    }

    function startAnim() {
        gameplaying = true;
    }


    function endGame() {
        gameplaying = false;

        musicSound2.stop();

        asteroidFade.start();
        asteroid2Fade.start();

        starFade.start();

        nyancat.visible = false;

        if (score >= highscore) {
            var res = Storage.setSetting("highscore", score);
            console.log("res: " + res);
        }

        gameOverTrigger.start();
    }

    SoundEffect {
        id: musicSound
        source: sfxPath + "game_music1.wav"
        volume: 0.75
        loops: 1

        onPlayingChanged: {
            console.log("onPlayingChanged " + playing + "; state: " + status);
            if (musicSound.playing == false)
                musicSound2.play();
        }
    }


    SoundEffect {
        id: gameOverSound
        source: sfxPath + "game_over.wav"
        volume: 0.85
        loops: 1
    }


    SoundEffect {
        id: musicSound2
        source: sfxPath + "game_music2.wav"
        volume: 0.75
        loops: 999
    }

    SoundEffect {
        id: rocketSound
        source: sfxPath + "rocket.wav"
    }

    SoundEffect {
        id: starSound
        source: sfxPath + "star.wav"
        volume: 0.6
    }

    SoundEffect {
        id: boomSound
        source: sfxPath + "boom.wav"
        volume: 0.7
    }


    Timer {
        id: gameOverTrigger
        interval: 1000;
        onTriggered: {
            gameOverLabel.visible = true;
            gameOverAnim.start();
            gameOverSound.play();
        }
    }

    Timer {
        interval: 300; running: !gamepaused; repeat: true
        onTriggered: {
            if (tailoffset > 0)
                tailoffset = -7;
            else
                tailoffset = 7;

        }
    }


    Timer {
        interval: 1000; running: gameplaying && !gamepaused; repeat: true
        onTriggered: {
            score = score + 1;
            if (highscore > 0 && score > highscore && newHighscoreLabel.visible && newHighscoreAnim.running == false) {
                console.log("new highscore: " + score);
                newHighscoreAnim.start();
            }
            if (score > highscore)
                highscore = score;
        }
    }

    Timer {
        interval: 20; running: gameplaying && !gamepaused; repeat: true
        onTriggered: {
            random = Math.random(123);
                asteroidCheck(asteroid, asteroidFade);
                asteroidCheck(asteroid2, asteroid2Fade);

                if (obstacleCheck(nyancat, star) && star.opacity == 1) {
                    stars = stars + 1;
                    star.opacity = 0.9;
                    starFade.start();
                    score += 5;
                    starSound.play();
                }

                // anim variable refresh - workaround
                if (starAnim.complete)
                    starAnim.start();

                if (asteroidAnim.complete)
                    asteroidAnim.start();

                if (score > 50 && asteroid2Anim.complete)
                    asteroid2Anim.start();
            }
    }


    function obstacleCheck(nyan, obstacle) {
        if (collisionCheck(nyan, obstacle.x, obstacle.y) ||
            collisionCheck(nyan, obstacle.x, obstacle.y + obstacle.height) )
            return true;
        else
            return false;
    }

    function collisionCheck(item, x, y) {
        var radius = 48;
        if (x>= item.x && x<=item.x+item.width && y>=item.y && y<= item.y+item.height)
            return true;
        else
            return false;
    }

    function asteroidCheck(asteroid, asteroidFade) {
        if (obstacleCheck(nyancat, asteroid) && asteroid.opacity === 1) {
            explosion.x = nyancat.x + 140;
            explosion.y = nyancat.y + 10;
            explosion.visible = true;
            explosionAnimation.start();
            asteroid.opacity = 0.9;
            asteroidFade.start();
            boomSound.play();

            if (lives == 0)
                endGame();
            else
                lives--;
        }
        if (obstacleCheck(asteroid, rocket) && rocket.x < 850 && asteroid.x < 850) {
            explosion.x = rocket.x;
            explosion.y = rocket.y;
            explosion.visible = true;
            explosionAnimation.start();
            asteroid.opacity = 0.9;
            asteroidFade.start();
            boomSound.play();

            score+=10;
        }
    }


    AnimatedImage {
        id: backgroundAnim
        paused: gamepaused
        source: gfxPath + "background.gif"
        height: 480;
        width: 854;
    }


    Image {
        id: stars1;
        source: gfxPath + "stars.png"
        height: 480;
        width: 854;
        x: 0

        SequentialAnimation on x {
            loops: Animation.Infinite
            paused: gamepaused
            PropertyAnimation { to: -854; duration: 1900 }
            PropertyAction { value: 0 }
        }
    }

    Image {
        id: stars2;
        source: gfxPath + "stars.png"
        height: 480;
        width: 854;
        x: 854

        SequentialAnimation on x {
            loops: Animation.Infinite
            paused: gamepaused
            PropertyAnimation { to: 0; duration: 1900 }
            PropertyAction { value: 854 }
        }
    }


    Image {
        id: explosion;
        source: gfxPath + "explosion.png"
        height: 96;
        width: 96;
        x: 20
        y: 20
        z: 10;
        visible: false;

        ParallelAnimation {
            id: explosionAnimation;
            PropertyAnimation { target: explosion; property: "scale"; from: 1; to: 4; duration: 1000; easing.type: Easing.OutQuad; }
            PropertyAnimation { target: explosion; property: "opacity"; from: 1; to: 0; duration: 1000; easing.type: Easing.InQuad; }
            PropertyAnimation { target: explosion; property: "x"; from: explosion.x; to: explosion.x-120; duration: 1000; easing.type: Easing.OutQuad; }
        }
    }


    Image {
        id: star;
        source: gfxPath + "star.png"
        height: 96;
        width: 96;
        x: 854
        y: 20

        SequentialAnimation {
            id: starAnim
            running: gameplaying
            paused: gamepaused
            //loops: Animation.Infinite
            ScriptAction {
               script: {
                   star.x = 854;
                   star.y = Math.random(111) * 400;
                   star.opacity = 1;
                   delayStar = 5000 + Math.random(222) * 10000;
               }
            }
            PauseAnimation { duration: delayStar }
            PropertyAnimation { target: star; property: "x"; to: -96; duration: 1800 }
        }

        SequentialAnimation {
            id: starFade
            loops: 1
            PropertyAnimation { target: star; property: "opacity"; to: 0; duration: 200 }
        }
    }


    function calcAsteroidDelay() {
        var aa = 1000 - score * 2;
        if (aa < 0)
            aa = 0;

        //debug.text = x;
        return 100 * Math.random(111);
    }

    Image {
        id: asteroid;
        source: gfxPath + "asteroid.png"
        height: 96;
        width: 96;
        x: 554
        y: 20
        visible: true;

        SequentialAnimation {
            id: asteroidAnim
            running: gameplaying && score > 0
            paused: gamepaused
            //loops: Animation.Infinite

            ScriptAction {
                script: {
                    asteroid.x = 854;
                    asteroid.y = Math.random(111) * 400;
                    asteroid.opacity = 1;
                    speed1 = 2000 - score * 3;
                    delay1 = random * 3000 - score * 4;
                    if (delay1 < 0)
                        delay1 = 0;
                    delay1 += 500;
                    console.log("speed1: " + speed1);
                    console.log("delay1: " + delay1);
                }
            }
            PauseAnimation { duration: delay1 }

            ParallelAnimation {
                PropertyAnimation { target: asteroid; property: "scale"; from: 0.5 + 2 * random * random; to: 1; duration: speed1 }
                PropertyAnimation { target: asteroid; property: "x"; from: 854; to: -96; duration: speed1 }
                PropertyAnimation { target: asteroid; property: "rotation"; from: 0; to: -60 + random * 120; duration: speed1 }
            }

        }

        SequentialAnimation {
            id: asteroidFade
            loops: 1
            PropertyAnimation { target: asteroid; property: "opacity"; to: 0; duration: 150 }
        }
    }

    Image {
        id: asteroid2;
        source: gfxPath + "asteroid.png"
        height: 96;
        width: 96;
        x: 854
        y: 20
        //visible: false;

        SequentialAnimation {
            id: asteroid2Anim
            running: gameplaying && score > 50
            paused: gamepaused
            //loops: Animation.Infinite
            ScriptAction {
                script: {
                    asteroid2.x = 854;
                    asteroid2.y = Math.random(111) * 400;
                    asteroid2.opacity = 1;
                    speed2 = 2200 - score * 3;
                    delay2 = random * 5000 - score * 4;
                    if (delay2 < 0)
                        delay2 = 0;
                    delay2 += 500;
                    console.log("speed2: " + speed2);
                    console.log("delay2: " + delay2);
                }
            }
            PauseAnimation { duration: delay2 }
            ParallelAnimation {
                PropertyAnimation { target: asteroid2; property: "scale"; from: 1 + 3 * random * random; to: 1; duration: speed2; easing.type: Easing.OutQuad; }
                PropertyAnimation { target: asteroid2; property: "x"; from: 854; to: -96; duration: speed2 }
                PropertyAnimation { target: asteroid2; property: "rotation"; from: 0; to: -60 + random * 120; duration: speed2 }
            }
        }

        SequentialAnimation {
            id: asteroid2Fade
            loops: 1
            PropertyAnimation { target: asteroid2; property: "opacity"; to: 0; duration: 150 }
        }
    }



    Image {
        anchors {
            horizontalCenter: parent.horizontalCenter
        }
        z: 20
        y: 4
        source: gfxPath + "pause_icon.png"
        opacity: 0.7
        width: 70
        height: 40

        MouseArea {
            anchors.fill: parent;
            onClicked: {
                gamepaused = !gamepaused;
            }
        }
    }

    /* ------------------------------------------------------------------------------ */

    Text {
        id: scoreLabel
        x: 10
        y: 4
        z: 20
        text: "SCORE: " + score
        color: "white"
        font.pixelSize: 32
        opacity: 0.8
    }

    Text {
        id: highscoreLabel
        x: 540
        y: 4
        z: 20
        text: "HIGHSCORE: " + highscore
        color: "white"
        font.pixelSize: 32
        opacity: 0.8
     }


    function fireRocket() {
        rocket.y = nyancat.y + 20;
        rocketAnimation.start();
        rocketSound.play();
    }

    onStarsChanged: {
        if (stars > 0)
            fireLabel.opacity = 0.5
        else
            fireLabel.opacity = 0;
    }

    Image {
        id: fireLabel
        x: parent.width - 150
        y: 340
        z: 20
        opacity: 0
        smooth: false
        source: gfxPath + "fire_button.png"

        // Touch events handling
        TouchEvents {
            onTouchEventReceived: {
                if (!gameplaying || gamepaused || stars == 0 || rocketAnimation.running)
                    return;

                var item = gamePage.childAt(x,y)
                // See if there is a function called play().
                if (item == fireLabel) {
                    fireRocket();
                    stars--;
                }
            }
        }

        Behavior on opacity {
            NumberAnimation { duration: 300 }
        }

     }


    Image {
        id: starLabel
        source: gfxPath + "star.png"
        width: 40;
        height: 40;
        x: 190
        y: 5
        z: 20
        opacity: 0.8
    }

    Text {
        id: starPoints
        anchors.left: starLabel.right
        anchors.top: starLabel.top
        x: 5
        z: 20
        text: stars
        color: "white"
        font.pixelSize: 32
        opacity: 0.8
     }


    Image {
        id: nyanLabel
        source: gfxPath + "nyan-cat-mini.png"
        width: 65;
        height: 40;
        //fillMode: Image.PreserveAspectFit
        x: 275
        y: 8
        z: 20
        opacity: 0.8
    }

    Text {
        anchors.left: nyanLabel.right
        y: 5
        x: 5
        z: 20
        text: lives;
        color: "white"
        font.pixelSize: 32
        opacity: 0.8
     }


    Image {
        id: rocket
        source: gfxPath + "rocket.png"
        height: 50
        width: 150
        y: 70
        x: 854

        ParallelAnimation {
            id: rocketAnimation;
            loops: 1
            PropertyAnimation { target: rocket; property: "x"; from: nyancat.x + 50; to: nyancat.x + 850; duration: 400; easing.type: Easing.InQuad; }
        }
    }


    AnimatedImage {
        id: nyancat
        playing: true
        paused: gamepaused
        source: gfxPath + "nyan-cat.gif"
        height: 140;
        fillMode: Image.PreserveAspectFit
        z: 2
        x: - 100
        visible: false;

        MouseArea {
            enabled: startLabelAnim.running || (gameplaying && !gamepaused)
            anchors.fill: parent
            drag.target: parent
            drag.axis: Drag.XandYAxis
            drag.minimumX: 0
            drag.maximumX: 640
            drag.minimumY: 0
            drag.maximumY: 360
        }

        SequentialAnimation {
            id: nyanCatAnim
            PropertyAction { target: nyancat; property: "x"; value: -200; }
            PropertyAction { target: nyancat; property: "visible"; value: true; }
            PauseAnimation { duration: 1000 }
            PropertyAnimation {
                target: nyancat;
                property: "x"
                to: 310;
                duration: 1500;
                easing.type: Easing.OutQuad;
            }
            PauseAnimation { duration: 300 }
            ScriptAction {
                script: startLabelAnim.start();
            }
            PauseAnimation { duration: 1000 }
            ScriptAction {
                script: startAnim();
            }
        }

    }

    Image {
        id: tail1
        source: gfxPath + "rainbow.png"
        parent: nyancat
        x: -6
        y: 20
        height: nyancat.height * 0.6
        z: -1

    }

    Image {
        id: tail2
        source: gfxPath + "rainbow.png"
        parent: tail1
        x: - width
        y: tailoffset
        height: nyancat.height * 0.6
    }

    Image {
        id: tail3
        source: gfxPath + "rainbow.png"
        parent: tail2
        x: - width
        y: - tailoffset
        height: nyancat.height * 0.6
        opacity: tail2.opacity * 0.9
    }

    Image {
        id: tail4
        source: gfxPath + "rainbow.png"
        parent: tail3
        x: - width
        y: - tailoffset
        height: nyancat.height * 0.6
        opacity: tail3.opacity * 0.9
    }

    Image {
        id: tail5
        source: gfxPath + "rainbow.png"
        parent: tail4
        x: - width
        y: tailoffset
        height: nyancat.height * 0.6
        opacity: tail4.opacity * 0.9
    }

    Image {
        id: tail6
        source: gfxPath + "rainbow.png"
        parent: tail5
        x: - width
        y: tailoffset
        height: nyancat.height * 0.6
        opacity: tail5.opacity * 0.9
    }

    Image {
        id: tail7
        source: gfxPath + "rainbow.png"
        parent: tail6
        x: - width
        y: - tailoffset
        height: nyancat.height * 0.6
        opacity: tail6.opacity * 0.9
    }

    Image {
        id: tail8
        source: gfxPath + "rainbow.png"
        parent: tail7
        x: - width
        y: - tailoffset
        height: nyancat.height * 0.6
        opacity: tail7.opacity * 0.9
    }

    Image {
        id: tail9
        source: gfxPath + "rainbow.png"
        parent: tail8
        x: - width
        y: tailoffset
        height: nyancat.height * 0.6
        opacity: tail8.opacity * 0.9
    }

    Image {
        id: tail10
        source: gfxPath + "rainbow.png"
        parent: tail9
        x: - width
        y: tailoffset
        height: nyancat.height * 0.6
        opacity: tail9.opacity * 0.9
    }

    Image {
        id: gameOverLabel
        source: gfxPath + "game_over.png"
        x: 200
        y: 200
        width: 450
        height: 80
        z: 10
        visible: false;


        MouseArea {
            anchors.fill: parent;
            onClicked: {
                gameOverSound.stop();
                showMenu();
            }
        }

        ParallelAnimation {
            id: gameOverAnim

            PropertyAnimation {
                target: gameOverLabel
                property: "opacity"
                from: 0.5;
                to: 1;
                duration: 2000;
                easing.type: Easing.OutCubic;
            }

            PropertyAnimation {
                target: gameOverLabel
                property: "scale"
                from: 0.3;
                to: 1;
                duration: 2000;
                easing.type: Easing.OutBounce;
            }
        }
    }




    Image {
        id: gamePausedLabel
        source: gfxPath + "game_paused.png"
        anchors.centerIn: parent
        width: 500
        height: 80
        z: 10
        opacity: 0


        MouseArea {
            anchors.fill: parent;
            onClicked: {
                gamepaused = false;
            }

        }

        ParallelAnimation {
            id: gamePausedAdnim

            PropertyAnimation {
                target: gamePausedLabel
                property: "opacity"
                from: 0.5;
                to: 1;
                duration: 2000;
                easing.type: Easing.OutCubic;
            }

            PropertyAnimation {
                target: gamePausedLabel
                property: "scale"
                from: 0.3;
                to: 1;
                duration: 2000;
                easing.type: Easing.OutBounce;
            }
        }
    }



    Image {
        id: newHighscoreLabel
        source: gfxPath + "new_highscore.png"
        x: 200
        y: 200
        width: 450
        height: 60
        z: 20
        opacity: 0;

        ParallelAnimation {
            id: newHighscoreAnim

            PropertyAnimation {
                target: newHighscoreLabel
                property: "opacity"
                from: 1;
                to: 0;
                duration: 1000;
                easing.type: Easing.InQuad;
            }

            PropertyAnimation {
                target: newHighscoreLabel
                property: "scale"
                from: 1;
                to: 3;
                duration: 1000;
                easing.type: Easing.InQuad
            }

            SequentialAnimation {
                PauseAnimation { duration: 1000; }
                ScriptAction {
                    script: newHighscoreLabel.visible = false;
                }
            }
        }
    }


    Image {
        id: startLabel
        source: gfxPath + "start.png"
        x: 300
        y: 200
        width: 250
        height: 60
        z: 20
        opacity: 0;

        ParallelAnimation {
            id: startLabelAnim

            PropertyAnimation {
                target: startLabel
                property: "opacity"
                from: 1;
                to: 0;
                duration: 1000;
                easing.type: Easing.InQuad;
            }

            PropertyAnimation {
                target: startLabel
                property: "scale"
                from: 1;
                to: 3;
                duration: 1000;
                easing.type: Easing.InQuad
            }

        }
    }

}
