import QtQuick 1.1
import com.nokia.meego 1.0
import QtMultimediaKit 1.1

Page {
    id: introPage

    property int tailoffset: 0;

    property bool nyanFly: false;

    orientationLock: PageOrientation.LockLandscape;


    Component.onCompleted: {
        startAnim.start();
    }


    onStatusChanged: {
        if (status == PageStatus.Deactivating)
            introMusic.stop();
    }

    Text {
        text: "v1.0.1"
        font.family: "Helvetica"
        font.pixelSize: 20
        color: "white"
        z: 10
        x: 10
        y: 10
        smooth: false
    }



    AnimatedImage {
        id: background
        paused: !active;
        source: gfxPath + "background.gif"
        height: 480;
        width: 854;
        opacity: 0;
    }

    Image {
        id: nyanLogoBig
        source: gfxPath + "nyan_logo.png"
        anchors {
            horizontalCenter: parent.horizontalCenter
        }
        y: 10
        width: 470
        height: 100
        opacity: 0;
    }

    Image {
        id: nyanLogoSmall
        source: gfxPath + "the_game.png"
        anchors {
            horizontalCenter: parent.horizontalCenter
        }
        y: 80
        z: 10
        width: 260
        height: 45
        opacity: 0;
    }

    Flipable {
        id: flipable
        anchors.fill:  parent
        z: 10
        opacity: 0

        property bool flipped: false

        front: menu
        back: help

        transform: Rotation {
                 id: rotation
                 origin.x: flipable.width/2
                 origin.y: flipable.height/2
                 axis.x: 0; axis.y: 1; axis.z: 0     // set axis.y to 1 to rotate around y-axis
                 angle: 0    // the default angle
        }

        states: State {
            PropertyChanges { target: rotation; angle: 180 }
            when: flipable.flipped
        }

        transitions: Transition {
            PropertyAnimation { target: rotation; property: "angle"; duration: 1000; easing.type: Easing.OutBack }
        }
   }



    Item {
        id: menu
        anchors {
            horizontalCenter: parent.horizontalCenter
        }

        Image {
            source: gfxPath + "start_game.png"
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            y: 250
            scale: 1.6
            z: 10

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    startGame();
                }
            }
        }

        Image {
            source: gfxPath + "help.png"
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            y: 340
            scale: 1.6
            z: 10

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    flipable.flipped = !flipable.flipped
                }
            }
        }

        Text {
            text: "Code: Sławomir Musiał\nIntro/ending music: Sławomir Musiał"
            font.family: "Helvetica"
            font.pixelSize: 20
            color: "white"
            z: 10
            height: 100
            x: 10 - 427
            y: 425
            smooth: false
        }

        Text {
            text: "Game music: Daniwell-Aidn\nNyan Cat graphics: prguitarman"
            font.family: "Helvetica"
            font.pixelSize: 20
            color: "white"
            z: 10
            height: 100
            y: 425
            x: 560 - 427
            width: 280
            smooth: false
            horizontalAlignment: Text.AlignRight
        }

    }


    Image {
        id: help
        source: gfxPath + "nyan_help.png"
        anchors {
            horizontalCenter: parent.horizontalCenter
        }
        y: 150

        MouseArea {
            anchors.fill: parent
            onClicked: flipable.flipped = !flipable.flipped
        }
    }



    Image {
        id: stars1;
        source: gfxPath + "stars.png"
        height: 480;
        width: 854;
        x: 0
        opacity: 0;

        SequentialAnimation on x {
            paused: !active;
            loops: Animation.Infinite
            PropertyAnimation { to: -854; duration: 1900 }
            PropertyAction { value: 0 }
        }
    }

    Image {
        id: stars2;
        source: gfxPath + "stars.png"
        height: 480;
        width: 854;
        x: 854
        opacity: 0;

        SequentialAnimation on x {
            paused: !active;
            loops: Animation.Infinite
            PropertyAnimation { to: 0; duration: 1900 }
            PropertyAction { value: 854 }
        }
    }



    AnimatedImage {
        id: nyancat
        source: gfxPath + "nyan-cat.gif"
        height: 140;
        paused: !active;
        fillMode: Image.PreserveAspectFit
        x: -200
        y: 300
        z: 2

        Behavior on y {
            SmoothedAnimation { velocity: 200; easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            PropertyAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }

    }

    Timer {
        interval: 1200; running: nyanFly && active; repeat: true
        onTriggered: {
            var y = Math.random(3) * 260 + 100;
            var x = Math.random(3) * 500 + 100;
            nyancat.y = y;
            nyancat.x = x;
            updateTail(x, y);
        }
    }

    function updateTail(x, y) {
        if (tailoffset > 0)
            tailoffset = -6;
        else
            tailoffset = 6;

        tail1.y = y + 20;
        tail1.x = x - 8;

        tail2.y = y + 20 + tailoffset;
        tail2.x = x - 8 - (46 * 1);

        tail3.y = y + 20 +  0 * tailoffset;
        tail3.x = x - 8 - (46 * 2);

        tail4.y = y + 20 - tailoffset;
        tail4.x = x - 8 - (46 * 3);

        tail5.y = y + 20;
        tail5.x = x - 8 - (46 * 4);

        tail6.y = y + 20 + tailoffset;
        tail6.x = x - 8 - (46 * 5);

        tail7.y = y + 20 - 0 * tailoffset;
        tail7.x = x - 8 - (46 * 6);

        tail8.y = y + 20 - tailoffset;
        tail8.x = x - 8 - (46 * 7);

        tail9.y = y + 20 + 1 * tailoffset;
        tail9.x = x - 8 - (46 * 8);

        tail10.y = y + 20 + 0 * tailoffset;
        tail10.x = x - 8 - (46 * 9);
    }

    function tailDelay(nr) {
        return 200 - nr * 10;
    }

    Image {
        id: tail1
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        z: 1

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(0); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail2
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        //opacity: 0.9

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(1); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail3
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail2.opacity * 0.9

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(2) ; easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail4
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail3.opacity * 0.9

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(3); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail5
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail4.opacity * 0.9


        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(4); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail6
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail5.opacity * 0.8


        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(5); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail7
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail6.opacity * 0.8

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(6); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail8
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail7.opacity * 0.7

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(7); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }
    }

    Image {
        id: tail9
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail7.opacity * 0.6

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(8); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }

    }

    Image {
        id: tail10
        source: gfxPath + "rainbow.png"
        x: -200
        y: 300
        height: nyancat.height * 0.6
        opacity: tail7.opacity * 0.4

        Behavior on y {
            SmoothedAnimation { velocity: tailDelay(9); easing.type: Easing.InOutQuad; }
        }
        Behavior on x {
            NumberAnimation { duration: 1000; easing.type: Easing.InOutQuad; }
        }

    }

    SequentialAnimation {
        id: startAnim;

        PauseAnimation { duration: 1000 }

        ScriptAction {
            script: introMusic.play();
        }

        ParallelAnimation {
            NumberAnimation {
                target: nyanLogoBig
                property: "scale"
                from: 3
                to: 1
                duration: 1700;
                easing.type: Easing.OutCubic;
            }

            NumberAnimation {
                target: nyanLogoBig
                property: "opacity"
                from: 0.2
                to: 1
                duration: 1700;
            }

            PropertyAnimation {
                target: stars1
                property: "opacity"
                from: 0;
                to: 1;
                duration: 1700;
                //easing.type: Easing.InCubic;
            }

            PropertyAnimation {
                target: stars2
                property: "opacity"
                from: 0;
                to: 1;
                duration: 1700;
                //easing.type: Easing.InCubic;
            }
        }


        ParallelAnimation {
            NumberAnimation {
                target: nyanLogoSmall
                property: "scale"
                from: 3
                to: 1
                duration: 1700;
                easing.type: Easing.OutCubic;
            }

            NumberAnimation {
                target: nyanLogoSmall
                property: "opacity"
                from: 0.2
                to: 1
                duration: 1700;
            }

            SequentialAnimation {
                PauseAnimation { duration: 500 }
                ScriptAction {
                    script: nyanFly = true;
                }
            }
        }


        //PauseAnimation { duration: 500 }

        NumberAnimation {
            target: background;
            property: "opacity"
            from: 0
            to: 1
            duration: 1000;
            easing.type: Easing.OutQuad;
        }


        //PauseAnimation { duration: 1000 }

        NumberAnimation {
            target: flipable;
            property: "opacity"
            from: 0
            to: 1
            duration: 500;
            easing.type: Easing.OutQuad;
        }
    }

}
