import QtQuick 1.1
import com.nokia.meego 1.0
import QtMultimediaKit 1.1
import "storage.js" as Storage

PageStackWindow {
    id: appWindow

    property string gfxPath:  "gfx/"
    property string sfxPath:  "sfx/"

    property bool active : true;

    initialPage: introPage

    GamePage {
        id: gamePage
    }

    IntroPage {
        id: introPage
    }

    platformStyle: ToolBarStyle { inverted:  true}

    showToolBar: false;
    showStatusBar: false;

    ToolBarLayout {
        id: commonTools
        visible: false;
        ToolIcon {
            platformIconId: "toolbar-view-menu"
            anchors.right: (parent === undefined) ? undefined : parent.right
            onClicked: (myMenu.status === DialogStatus.Closed) ? myMenu.open() : myMenu.close()
        }
    }


    states: [
            State {
                name: "fullsize-visible"
                when: platformWindow.viewMode == WindowState.Fullsize && platformWindow.visible
                StateChangeScript {
                    script: {
                        //if (pageStack.currentPage == standbyPage) { pageStack.pop() }
                        //console.log("Visibility: Fullsize and visible!")
                        active = true;
                        if (pageStack.currentPage == introPage && introPage.nyanFly)
                            introMusic.play();
                    }
                }
            },
            State {
                name: "thumbnail-or-invisible"
                when: platformWindow.viewMode == WindowState.Thumbnail || !platformWindow.visible
                StateChangeScript {
                    script: {
                        //if (pageStack.currentPage == mainPage) { pageStack.push(standbyPage) }
                        //console.log("Visibility: Not fullsize (Thumbnail and/or invisible)")
                        active = false;
                        if (pageStack.currentPage == introPage)
                            introMusic.stop();
                        if (pageStack.currentPage == gamePage && gamePage.gameplaying)
                            gamePage.gamepaused = true;
                    }
                }
            }
    ]





    SoundEffect {
        id: introMusic
        source: sfxPath + "intro_music.wav"
        volume: 0.95
        loops: 999
    }


    function startGame() {
        pageStack.push(gamePage);
        gamePage.startGame();
        active = false;
        screen.allowSwipe = false;
    }

    function showMenu() {
        introMusic.play();
        pageStack.pop();
        active = true;
        screen.allowSwipe = true;
    }

}
